#ifndef CMDSTAT_H__INCLUDED
#define CMDSTAT_H__INCLUDED
struct scsi_cmd;
struct scsi_lu;
struct concat_buf;

#include <stdbool.h>
#include <stdint.h>

#define CMDSTAT_HISTORY_SECONDS (15*60)
#define CMDSTAT_SAMPLE_SECONDS 5
#define CMDSTAT_NBUF (CMDSTAT_HISTORY_SECONDS/CMDSTAT_SAMPLE_SECONDS)

/* scsi_cmd lifecycle has some (5) points whose time we save to
 * calculate latencies */

enum cmd_life_event {
	CMDLIFE_RX_START = 0,
	CMDLIFE_QUEUED,
	CMDLIFE_RUNNING,
	CMDLIFE_DONE_DEVICE,
	CMDLIFE_DONE_TX,
	CMDLIFE_LIMIT
};

enum cmd_latency_type {
	CMDCNT_READOP_QRLAT = 0,
	CMDCNT_READOP_DEVLAT,
	CMDCNT_READOP_TXLAT,
	CMDCNT_WRITEOP_RXLAT,
	CMDCNT_WRITEOP_QRLAT,
	CMDCNT_WRITEOP_DEVLAT,
	CMDCNT_LIMIT
};

enum cmd_direction {
	CMDDIR_READ = 0,
	CMDDIR_WRITE,
	CMDDIR_LIMIT
};

enum dev_queue {
	DEVQUEUE_WAITING = 0,
	DEVQUEUE_INFLIGHT,
	DEVQUEUE_LIMIT
};

enum proc_usage {
	PROCUSAGE_MAXRSS = 0,
	PROCUSAGE_CPUTIME,
	PROCUSAGE_LIMIT,
};

struct life_events {
	struct timespec ts[CMDLIFE_LIMIT];
};

struct stat_latency {
	uint32_t ncommands;
	uint64_t latsum;
	uint64_t latmax;
};

struct stat_queue {
	uint32_t nentries;
	uint32_t depthsum;
	uint32_t depthmax;
};

struct stat_xfer {
	uint32_t ncommands;
	uint64_t total_length;
};

struct stat_usage {
	uint32_t nentries;
	uint32_t valuesum;
	uint32_t valuemax;
};

struct stat_entry {
	struct stat_latency lat[CMDCNT_LIMIT];
	struct stat_xfer xfer[CMDDIR_LIMIT];
	struct stat_queue queue[DEVQUEUE_LIMIT];
	struct stat_usage usage[PROCUSAGE_LIMIT];
};

struct stat_history {
	struct stat_entry ent[CMDSTAT_NBUF];
	uint32_t last_nbuf;
};

void cmd_track_time(struct scsi_cmd* cmd, enum cmd_life_event what);
void dev_track_queue(struct scsi_lu* dev, enum dev_queue what, uint32_t depth);
void cmdstat_device_print(struct concat_buf *b, struct scsi_lu *dev);
void cmdstat_alldev_print(struct concat_buf *b);
void cmdstat_init(void);
uint64_t ts_valid_diff(const struct timespec *start, const struct timespec *end);

struct target;
void device_print_json(struct target *target, struct concat_buf *b, struct scsi_lu *lu);

#endif	/* CMDSTAT_H__INCLUDED */
